-- initialize three rings:
-- R for the variables of the cubic
-- A for the coefficients of a cubic
-- S for the free parameter on a pencil

R = QQ[x_0..x_2]
A = QQ[apply(compositions(3,3), iii-> a_iii)]
S = QQ[s]
AR = A**R**S

-- initialize vectors for the x and a variables
-- and a general polynomial fgen with variable coefficients
xx = sub(vars(R),AR)
aa = sub(vars(A),AR)

fgen = 6*sum(compositions(3,3), iii-> (
	a_iii * product(3, j-> 1/(iii_j)!*x_j^(iii_j))))

-- the file cubicInvs.m2 contains a hardcoded expression 
-- of the discriminant of plane cubics; we use it to define
-- a handy function that evaluates it.
load "SupportingFiles/cubicInvs.m2"

disc = ff -> (
    apply(compositions(3,3), cc -> (
	    mm = product(3, j-> x_j^(cc_j));
	    ee_cc = diff(mm,ff)));
    Disc())

-- we compute the tangent cone in every case of the 
-- classification of plane cubics. For each orbit:
-- f is the pencil of cubics generated by the singular cubic
-- and the generic cubic;
-- D is the discriminant evaluate at f: it is a univariate polynomial in s 
-- whose coefficients depend on the variables a;
-- m is the 'guessed multiplicity': we verify that the multiplicity of the
-- singular cubic is indeed m;
-- for j from 0 to m-1, we verify that the coefficient of s^j in D is 0;
-- we display the m-th coefficient: the user should observe that indeed
-- it coincides with the equation of the claimed tangent cone in the text.

myfile = "result_tangentConesPlaneCubics.txt";

myfile << "For each singular cubic, we compute the generators of the tangent cone in the coordinates
a_{i,j,k} of a general cubic \n";
myfile << (toString(fgen) | "\n \n");

--- generic nodal cubic:
f0 = x_2^2 * x_0 - x_1^2*(x_1+x_0) 
f = f0 + s * fgen 

D = disc(f);

m = 1
apply(m, j-> sub(diff(s^j,D),s=>0))
TCf = factor (ideal mingens ideal sub(diff(s^m,D), s=>0))_0


myfile << "The NODAL CUBIC f = "|toString (f0)|" is singular at (1,0,0). \n"
myfile << "The ideal of the tangent cone T_f Disc is generated by "|(toString(TCf))|"\n \n"

--- cuspidal cubic
f0 = x_2^2*x_0 - x_1^3 
f = f0 + s * fgen 

D = disc(f);

m = 2
apply(m, j-> sub(diff(s^j,D),s=>0))
TCf = factor (ideal mingens ideal sub(diff(s^m,D), s=>0))_0


myfile << "The CUSPIDAL CUBIC f = "|toString (f0)|" is singular at (1,0,0). \n"
myfile << "The ideal of the tangent cone T_f Disc is generated by "|(toString(TCf))|"\n \n"


--- generic reducible cubic ( conic + secant line)
f0 = x_0 * (x_0^2 + x_1*x_2) 
f = f0 + s * fgen 

D = disc(f);

m = 2
apply(m, j-> sub(diff(s^j,D),s=>0))
TCf = factor (ideal mingens ideal sub(diff(s^m,D), s=>0))_0


myfile << "The CONIC+SECANT f = "|toString (f0)|" is singular at (0,1,0) and (0,0,1). \n"
myfile << "The ideal of the tangent cone T_f Disc is generated by "|(toString(TCf))|"\n \n"


--- degenerate reducible cubic (conic + tangent line)
f0 = x_0 * (x_0 * x_1 + x_2^2)
f = f0 + s * fgen 

D = disc(f);

m = 3
apply(m, j-> sub(diff(s^j,D),s=>0))
TCf = factor (ideal mingens ideal sub(diff(s^m,D), s=>0))_0

myfile << "The CONIC+TANGENT f = "|toString (f0)|" is singular at (0,1,0). \n"
myfile << "The ideal of the tangent cone T_f Disc is generated by "|(toString(TCf))|"\n \n"

--- triangles (product of three linearly independent lines)
f0 = x_0 * x_1*x_2 
f = f0 + s * fgen 

D = disc(f);

m = 3
apply(m, j-> sub(diff(s^j,D),s=>0))
TCf = factor (ideal mingens ideal sub(diff(s^m,D), s=>0))_0

myfile << "The TRIANGLE f = "|toString (f0)|" is singular at (1,0,0),(0,1,0),(0,0,1). \n"
myfile << "The ideal of the tangent cone T_f Disc is generated by "|(toString(TCf))|"\n \n"


--- asterisk (product of three lines, not linearly independent)
f0 = x_0*x_1*(x_0+x_1)
f = f0 + s * fgen 

D = disc(f);

m = 4
apply(m, j-> sub(diff(s^j,D),s=>0))
TCf = factor (ideal mingens ideal sub(diff(s^m,D), s=>0))_0

myfile << "The ASTERISK f = "|toString (f0)|" is singular at (0,0,1). \n"
myfile << "The ideal of the tangent cone T_f Disc is generated by "|(toString(TCf))|"\n \n"

--- line and double line 
f0 = x_0^2 * x_1
f = f0 + s * fgen 

D = disc(f);

m = 6
apply(m, j-> sub(diff(s^j,D),s=>0))
TCf = factor( (ideal mingens ideal sub(diff(s^m,D), s=>0))_0)

myfile << "The LINE+DOUBLELINE f = "|toString (f0)|" is singular on the line x_0=0, with an embeddded point at (0,0,1). \n"
myfile << "The ideal of the tangent cone T_f Disc is generated by "|(toString(TCf))|"\n \n"

--- triple line
f0 = x_0^3 
f = f0 + s * fgen 

D = disc(f);

m = 8
apply(m, j-> sub(diff(s^j,D),s=>0))
TCf = factor( (ideal mingens ideal sub(diff(s^m,D), s=>0))_0)

myfile << "The TRIPLELINE f = "|toString (f0)|" is singular on the line x_0=0. \n"
myfile << "The ideal of the tangent cone T_f Disc is generated by "|(toString(TCf))|"\n \n"

myfile << close
